package com.aliyun.odps.spark.examples.makedata

import com.aliyun.odps.spark.examples.myUtils.{MyHdfsUtils, ParamUtils}
import org.apache.hadoop.io.compress.GzipCodec
import org.apache.spark.sql.SparkSession
import scala.collection.JavaConversions._
import scala.collection.mutable.ArrayBuffer
import scala.io.Source
/*

 */

object makedata_15_bucket_20240608 {
  def main(args: Array[String]): Unit = {

    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    val loader = getClass.getClassLoader
    val resourceUrl = loader.getResource("20240608_feature_name.txt")
    val content =
      if (resourceUrl != null) {
        val content = Source.fromURL(resourceUrl).getLines().mkString("\n")
        Source.fromURL(resourceUrl).close()
        content
      } else {
        ""
      }
    println(content)
    val contentList = content.split("\n")
      .map(r=> r.replace(" ", "").replaceAll("\n", ""))
      .filter(r=> r.nonEmpty).toList



    // 1 读取参数
    val param = ParamUtils.parseArgs(args)
    val readPath = param.getOrElse("readPath", "/dw/recommend/model/14_feature_data/20240607")
    val savePath = param.getOrElse("savePath", "/dw/recommend/model/15_bucket_data/")
    val fileName = param.getOrElse("fileName", "20240607_200")
    val sampleRate = param.getOrElse("sampleRate", "0.1").toDouble
    val bucketNum = param.getOrElse("bucketNum", "200").toInt

    val data = sc.textFile(readPath)
    val data1 = data.map(r => {
      val rList = r.split("\t")
      val doubles = rList(2).split(",").map(_.toDouble)
      doubles
    }).sample(false, sampleRate ).repartition(20)

    val result = new ArrayBuffer[String]()

    for (i <- contentList.indices){
      println("特征:" + contentList(i))
      val data2 = data1.map(r => r(i)).filter(_ > 1E-8).collect().sorted
      val len = data2.length
      val oneBucketNum = (len - 1) / (bucketNum - 1) + 1 // 确保每个桶至少有一个元素
      val buffers = new ArrayBuffer[Double]()

      var lastBucketValue = data2(0) // 记录上一个桶的切分点
      for (j <- 0 until len by oneBucketNum) {
        val d = data2(j)
        if (j > 0 && d != lastBucketValue) {
          // 如果当前切分点不同于上一个切分点，则保存当前切分点
          buffers += d
        }
        lastBucketValue = d // 更新上一个桶的切分点
      }

      // 最后一个桶的结束点应该是数组的最后一个元素
      if (!buffers.contains(data2.last)) {
        buffers += data2.last
      }
      result.add(contentList(i) + "\t" + bucketNum.toString + "\t" + buffers.mkString(","))
    }
    val data3 = sc.parallelize(result)


    // 4 保存数据到hdfs
    val hdfsPath = savePath + "/" + fileName
    if (hdfsPath.nonEmpty && hdfsPath.startsWith("/dw/recommend/model/")) {
      println("删除路径并开始数据写入:" + hdfsPath)
      MyHdfsUtils.delete_hdfs_path(hdfsPath)
      data3.repartition(1).saveAsTextFile(hdfsPath, classOf[GzipCodec])
    } else {
      println("路径不合法，无法写入:" + hdfsPath)
    }
  }
}
